# -*- coding: utf-8 -*-
# FreeCAD Exploded Assembly Animation Workbench
# (c) 2016 Javier Martinez Garcia

#***************************************************************************
#*   (c) Javier Martinez Garcia 2016                                       *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License (GPL)            *
#*   as published by the Free Software Foundation; either version 2 of     *
#*   the License, or (at your option) any later version.                   *
#*   for detail see the LICENCE text file.                                 *
#*                                                                         *
#*   This macro is distributed in the hope that it will be useful,         *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU Lesser General Public License for more details.                   *
#*                                                                         *
#*   You should have received a copy of the GNU Library General Public     *
#*   License along with FreeCAD; if not, write to the Free Software        *
#*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#*   USA                                                                   *
#*                                                                         *
#***************************************************************************/

import os
import FreeCAD
import FreeCADGui
import ExplodedAssembly as ea
import CameraAnimation as ca
__dir__ = os.path.dirname(__file__)

# TODO CHANGE NAME: SIMPLE DISASSEMBLE GROUP, WIRE DISASSEMBLE GROUP


class CreateBoltGroup:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/BoltGroup.svg',
                'MenuText': 'ボルトグループの作成',
                'ToolTip': 'ねじ、ナット、ボルト用の特別な分解グループを作成してください... \n アニメートするシェイプの円形エッジを選択し、\n 選択したシェイプを移動する方向に\n法線ベクトルを持つ面 (任意のシェイプ) を 1 つ選択します'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        ea.checkDocumentStructure()
        ea.createBoltDisassemble()


class CreateSimpleGroup:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/SimpleGroup.svg',
                'MenuText': '単純なグループの作成',
                'ToolTip': '分解するオブジェクトを選択し、\n最後にその法線が軌道ディレクター ベクトルである面を選択します'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        ea.checkDocumentStructure()
        ea.createSimpleDisassemble()

class CreateWireGroup:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/WireTrajectory.svg',
                'MenuText': 'ルートの作成',
                'ToolTip': '最初に 1 つまたは複数の形状を選択し、最後に軌道を表すワイヤまたはスケッチを選択します'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        pass



class PlaceBeforeSelectedTrajectory:
    # execute this function with caution
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/PlaceBeforeTrajectory.svg',
                'MenuText': '場所変更前',
                'ToolTip': '再割り当てする軌道を (順番に) 選択し、最後に\n配置する前の軌道を選択します'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        # check selection before running (rearrange objects is dangerous, use with caution)
        try:
            sel = FreeCAD.Gui.Selection.getSelectionEx()
            a = sel[0].Object.Distance
            a = sel[1].Object.Distance
            #  selection are trajectories, proceed
            ea.placeBeforeSelectedTrajectory()
            ea.resetPlacement()
            FreeCAD.Gui.Selection.clearSelection()
            FreeCAD.Gui.Selection.addSelection(sel[0].Object)
            ea.goToSelectedTrajectory()

        except:
            FreeCAD.Console.PrintError('\n Select exploded assembly trajectory objects only')

class ModifyIndividualObjectTrajectory:
        # execute this function with caution
        def GetResources(self):
            return {'Pixmap': __dir__ + '/icons/ModifyIndividualObjectTrajectory.svg',
                    'MenuText': '個々のオブジェクトの軌道を変更する',
                    'ToolTip': 'オブジェクトを一度に複数の方向に分解します\n軌道を選択し、オブジェクトと\nその法線が新しいディレクタベクトルである面を選択します'}

        def IsActive(self):
            if FreeCADGui.ActiveDocument:
                if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                    return True

            else:
                return False

        def Activated(self):
            # check selection before running (rearrange objects is dangerous, use with caution)
            ea.modifyIndividualObjectTrajectory()
            sel_traj = FreeCAD.Gui.Selection.getSelectionEx()[0].Object
            FreeCAD.Gui.Selection.clearSelection()
            FreeCAD.Gui.Selection.addSelection(sel_traj)
            ea.goToSelectedTrajectory()


# camera commands
class CreateManualCamera:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/AnimationCameraManual.svg',
                'MenuText': '手動アニメーションカメラ',
                'ToolTip': '2 つのカメラ間のトランジションを作成する'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        pass


class CreateEdgeCamera:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/AnimationCameraEdge.svg',
                'MenuText': '手動アニメーションカメラ',
                'ToolTip': '2 つのカメラ間のトランジションを作成する'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        pass


class CreateFollowCamera:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/AnimationCameraFollow.svg',
                'MenuText': '手動アニメーションカメラ',
                'ToolTip': '2 つのカメラ間のトランジションを作成する'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        pass


class PlayForward:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/PlayForward.svg',
                'MenuText': '再生',
                'ToolTip': 'アセンブリ アニメーションの実行'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        EA = FreeCAD.ActiveDocument.ExplodedAssembly
        if EA.CurrentTrajectory <= 0:
            # if exploded state = 0 or -1, reset and run
            ea.resetPlacement()
            ea.runAnimation()

        else:
            # if animation has been paused in the middle:
            cr_traj = EA.Group[EA.CurrentTrajectory]
            ea.runAnimation(start=EA.CurrentTrajectory+1, mode='toPoint')


class PlayBackward:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/PlayBackward.svg',
                'MenuText': '逆再生',
                'ToolTip': __dir__ + 'icons/TrajectoryEdit.svg'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        EA = FreeCAD.ActiveDocument.ExplodedAssembly
        if EA.CurrentTrajectory <= 0:
            # if exploded state = 0 or -1, reset and run
            ea.resetPlacement()
            ea.goToEnd()
            ea.runAnimation(direction='backward')

        else:
            # if animation has been paused in the middle:
            ea.runAnimation(end=-EA.CurrentTrajectory - 1 + len(EA.Group),
                            mode='toPoint',
                            direction='backward')


class StopAnimation:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/Pause.svg',
                'MenuText': 'アニメーションを停止',
                'ToolTip': '現在の軌道でアニメーションを停止します'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return False

            else:
                return True

        else:
            return False

    def Activated(self):
        FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation = False
        # FreeCAD.Gui.updateGui()


class GoToStart:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/GoToStart.svg',
                'MenuText': '組み立て ',
                'ToolTip': 'パーツの組み立て位置へ移動'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument.ExplodedAssembly:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        ea.resetPlacement()
        FreeCAD.ActiveDocument.ExplodedAssembly.CurrentTrajectory = 0


class GoToEnd:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/GoToEnd.svg',
                'MenuText': '分解する',
                'ToolTip': 'すべての軌道を展開する'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument.ExplodedAssembly:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        ea.resetPlacement()
        ea.goToEnd()
        FreeCAD.ActiveDocument.ExplodedAssembly.CurrentTrajectory = -1


class GoToSelectedTrajectory:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/ExplodeToSelection.svg',
                'MenuText': '選択範囲を展開',
                'ToolTip': '選択した軌道まで展開'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument.ExplodedAssembly:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        try:
            ea.resetPlacement()
            ea.goToSelectedTrajectory()

        except:
            FreeCAD.Console.PrintError('Error: Select one exploded animation trajectory')


class ToggleTrajectoryVisibility:
    def __init__(self):
        self.visibility = True

    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/TrajectoryLineVisibility.svg',
                'MenuText': '軌道の可視性',
                'ToolTip': '軌道の表示設定の切り替え'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument.ExplodedAssembly:
            return True

        else:
            return False

    def Activated(self):
        self.visibility = not(self.visibility)
        ea.visibilityTrajectoryLines(self.visibility)



# Assembly tools ###############################################################
class AlignToEdge:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/AlignToEdge.svg',
                'MenuText': '端に揃える',
                'ToolTip': '図形を位置合わせするための補助ツール。\n位置合わせするオブジェクトの 1 つのエッジを選択し、\n基準として使用されるオブジェクトのエッジを選択します'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        sel = FreeCAD.Gui.Selection.getSelectionEx()
        objA = sel[0].Object
        edgeA = sel[0].SubObjects[0]
        edgeB = sel[1].SubObjects[0]
        # transform object A placement
        # edge vector
        va = (edgeA.Curve.EndPoint - edgeA.Curve.StartPoint).normalize()
        vb = (edgeB.Curve.EndPoint - edgeB.Curve.StartPoint).normalize()
        # rot centre
        centre = edgeA.Curve.StartPoint
        # new placement
        new_plm = FreeCAD.Placement(FreeCAD.Vector(0,0,0), FreeCAD.Rotation(va, vb), centre)
        # apply placement
        objA.Placement = new_plm.multiply(objA.Placement)


class Rotate15:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/Rotate90.svg',
                'MenuText': '回転 15',
                'ToolTip': '図形を 15 度回転する補助ツールです。\n回転するオブジェクトの面を選択すると、\nその法線を回転軸として使用して 15 度回転されます\n'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        sel = FreeCAD.Gui.Selection.getSelectionEx()
        objA = sel[0].Object
        selFace = sel[0].SubObjects[0]
        rot_center = selFace.CenterOfMass
        rot_axis = selFace.normalAt(0, 0)
        rot = FreeCAD.Rotation(rot_axis, 15)
        objA.Placement = FreeCAD.Placement(FreeCAD.Vector(0,0,0), rot, rot_center).multiply(objA.Placement)

class PointToPoint:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/SharePoint.svg',
                'MenuText': 'ポイントからポイントへ',
                'ToolTip': '点から点へ移動するための補助ツール。\n移動する図形から 1 つの点を選択し、\n 次に、それを配置する他の図形の点を選択します。'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        sel = FreeCAD.Gui.Selection.getSelectionEx()
        objA = sel[0].Object
        PA = sel[0].SubObjects[0]
        PB = sel[1].SubObjects[0]
        v = PB.Point - PA.Point
        objA.Placement.Base.x += v.x
        objA.Placement.Base.y += v.y
        objA.Placement.Base.z += v.z


class PlaceConcentric:
    def GetResources(self):
        return {'Pixmap': __dir__ + '/icons/ShareCenter.svg',
                'MenuText': '同心円状に配置',
                'ToolTip': '2 つの図形を同心円状に配置する補助ツール\n選択した最初の円形エッジを、選択した 2 番目の円形エッジと同心円状に配置します'}

    def IsActive(self):
        if FreeCADGui.ActiveDocument:
            if not(FreeCAD.ActiveDocument.ExplodedAssembly.InAnimation):
                return True

        else:
            return False

    def Activated(self):
        sel = FreeCAD.Gui.Selection.getSelectionEx()
        objA = sel[0].Object
        CentreA = sel[0].SubObjects[0].Curve.Center
        CentreB = sel[1].SubObjects[0].Curve.Center
        v = CentreB - CentreA
        objA.Placement.Base.x += v.x
        objA.Placement.Base.y += v.y
        objA.Placement.Base.z += v.z


# non icon comands
class LoadExampleFile:
    def GetResources(self):
        return {'Pixmap': '',
                'MenuText': 'サンプルファイルのロード',
                'ToolTip': '分解アセンブリのサンプルファイルをロードする'}

    def IsActive(self):
        if not(FreeCADGui.ActiveDocument):
            return True

        else:
            if FreeCAD.ActiveDocument.Name != 'example':
                return True

            else:
                return False

    def Activated(self):
        FreeCAD.open(__dir__ + '/example.fcstd')

if FreeCAD.GuiUp:
    FreeCAD.Gui.addCommand('CreateBoltGroup', CreateBoltGroup())
    FreeCAD.Gui.addCommand('CreateSimpleGroup', CreateSimpleGroup())
    FreeCAD.Gui.addCommand('CreateWireGroup', CreateWireGroup())
    FreeCAD.Gui.addCommand('ModifyIndividualObjectTrajectory', ModifyIndividualObjectTrajectory())
    FreeCAD.Gui.addCommand('CreateManualCamera', CreateManualCamera())
    FreeCAD.Gui.addCommand('CreateEdgeCamera', CreateEdgeCamera())
    FreeCAD.Gui.addCommand('CreateFollowCamera', CreateFollowCamera())
    FreeCAD.Gui.addCommand('PlaceBeforeSelectedTrajectory', PlaceBeforeSelectedTrajectory())
    FreeCAD.Gui.addCommand('GoToStart', GoToStart())
    FreeCAD.Gui.addCommand('PlayBackward', PlayBackward())
    FreeCAD.Gui.addCommand('StopAnimation', StopAnimation())
    FreeCAD.Gui.addCommand('PlayForward', PlayForward())
    FreeCAD.Gui.addCommand('GoToEnd', GoToEnd())
    FreeCAD.Gui.addCommand('GoToSelectedTrajectory',GoToSelectedTrajectory())
    FreeCAD.Gui.addCommand('ToggleTrajectoryVisibility', ToggleTrajectoryVisibility())
    FreeCAD.Gui.addCommand('AlignToEdge', AlignToEdge())
    FreeCAD.Gui.addCommand('Rotate15', Rotate15())
    FreeCAD.Gui.addCommand('PointToPoint', PointToPoint())
    FreeCAD.Gui.addCommand('PlaceConcentric', PlaceConcentric())
    FreeCAD.Gui.addCommand('LoadExampleFile', LoadExampleFile())
